from honeybee_schema.radiance.modifier import Plastic, Glass, BSDF, Glow, Light, \
    Trans, Mirror, Metal
from copy import copy
from pydantic import ValidationError
import pytest
import os
import json

# target folder where all of the samples live
root = os.path.dirname(os.path.dirname(__file__))
target_folder = os.path.join(root, 'samples', 'modifier')


def test_modifier_plastic_black():
    file_path = os.path.join(target_folder, 'modifier_plastic_black.json')
    Plastic.parse_file(file_path)


def test_modifier_plastic_generic_ceiling():
    file_path = os.path.join(target_folder, 'modifier_plastic_generic_ceiling.json')
    Plastic.parse_file(file_path)


def test_modifier_plastic_generic_wall():
    file_path = os.path.join(target_folder, 'modifier_plastic_generic_wall.json')
    Plastic.parse_file(file_path)


def test_plastic_wrong():
    file_path = os.path.join(target_folder, 'modifier_plastic_black.json')
    with open(file_path) as json_file:
        plastic_modifier = json.load(json_file)

    # Test wrong type
    plastic_modifier_test = copy(plastic_modifier)
    plastic_modifier_test["type"] = 'glass'
    with pytest.raises(ValidationError):
        Plastic.parse_obj(plastic_modifier_test)

    # Test illegal character in identifier
    plastic_modifier_test = copy(plastic_modifier)
    plastic_modifier_test["identifier"] = '$#^&///'
    with pytest.raises(ValidationError):
        Plastic.parse_obj(plastic_modifier_test)

    # Test reflectance range
    plastic_modifier_test = copy(plastic_modifier)
    plastic_modifier_test["r_reflectance"] = -1
    with pytest.raises(ValidationError):
        Plastic.parse_obj(plastic_modifier_test)


def test_modifier_metal():
    file_path = os.path.join(target_folder, 'modifier_metal.json')
    Metal.parse_file(file_path)


def test_glass_generic_exterior_window():
    file_path = os.path.join(target_folder,
                             'modifier_glass_generic_exterior_window.json')
    Glass.parse_file(file_path)


def test_glass_air_boundary():
    file_path = os.path.join(target_folder,
                             'modifier_glass_air_boundary.json')
    Glass.parse_file(file_path)


def test_mirror_typical():
    file_path = os.path.join(target_folder, 'modifier_mirror_typical.json')
    Mirror.parse_file(file_path)

def test_mirror_invisible():
    file_path = os.path.join(target_folder, 'modifier_mirror_invisible.json')
    Mirror.parse_file(file_path)


def test_bsdf_klemsfull():
    file_path = os.path.join(target_folder, 'modifier_bsdf_klemsfull.json')
    BSDF.parse_file(file_path)


def test_modifier_glow_white():
    file_path = os.path.join(target_folder, 'modifier_glow_white.json')
    Glow.parse_file(file_path)


def test_modifier_light_green_spotlight():
    file_path = os.path.join(target_folder, 'modifier_light_green_spotlight.json')
    Light.parse_file(file_path)


def test_trans_tree_foliage():
    file_path = os.path.join(target_folder, 'modifier_trans_tree_foliage.json')
    Trans.parse_file(file_path)


def test_trans_tree_foliage_wrong():
    file_path = os.path.join(target_folder, 'modifier_trans_tree_foliage.json')
    with open(file_path) as json_file:
        trans_modifier = json.load(json_file)

    # Test wrong sum
    trans_modifier_test = copy(trans_modifier)
    trans_modifier_test["transmitted_diff"] = 0.6
    trans_modifier_test["transmitted_spec"] = 0.6
    with pytest.raises(ValidationError):
        Trans.parse_obj(trans_modifier_test)




"""SensorGrid and Sensor Schema"""
from pydantic import Field, constr
from typing import List
from enum import Enum
from .._base import NoExtraBaseModel
from ._base import IDdRadianceBaseModel


class _RadianceAsset(IDdRadianceBaseModel):
    """Hidden base class for all Radiance Assets."""

    room_identifier: str = Field(
        None,
        regex=r'[A-Za-z0-9_-]',
        min_length=1,
        max_length=100,
        description='Optional text string for the Room identifier to which this '
        'object belongs. This will be used to narrow down the number of '
        'aperture groups that have to be run with this sensor grid. If None, '
        'the grid will be run with all aperture groups in the model.'
    )

    light_path: List[List[str]] = Field(
        None,
        description='Get or set a list of lists for the light path from the object to '
        'the sky. Each sub-list contains identifiers of aperture groups through which '
        'light passes. (eg. [["SouthWindow1"], ["static_apertures", "NorthWindow2"]]).'
        'Setting this property will override any auto-calculation of the light '
        'path from the model and room_identifier upon export to the simulation.'
    )


class Sensor(NoExtraBaseModel):
    """A single Radiance of sensors."""

    type: constr(regex='^Sensor$') = 'Sensor'

    pos: List[float] = Field(
        ...,
        description="Position of sensor in space as an array of (x, y, z) values.",
        min_items=3,
        max_items=3
    )

    dir: List[float] = Field(
        ...,
        description="Direction of sensor as an array of (x, y, z) values.",
        min_items=3,
        max_items=3
    )


class SensorGrid(_RadianceAsset):
    """A grid of sensors."""

    type: constr(regex='^SensorGrid$') = 'SensorGrid'

    sensors: List[Sensor] = Field(
        ...,
        description="A list of sensors that belong to the grid."
    )


class ViewType(str, Enum):
    """A single character for the view type (-vt)."""
    perspective = 'v'
    hemispherical_fisheye = 'h'
    parallel = 'l'
    cylindrical_panorama = 'c'
    angular_fisheye = 'a'
    planisphere = 's'


class View(_RadianceAsset):
    """A single Radiance of sensors."""

    type: constr(regex='^View$') = 'View'

    position: List[float] = Field(
        ...,
        description='The view position (-vp) as an array of (x, y, z) values.'
        'This is the focal point of a perspective view or the center of a '
        'parallel projection.',
        min_items=3,
        max_items=3
    )

    direction: List[float] = Field(
        ...,
        description='The view direction (-vd) as an array of (x, y, z) values.'
            'The length of this vector indicates the focal distance as needed by '
            'the pixel depth of field (-pd) in rpict.',
        min_items=3,
        max_items=3
    )

    up_vector: List[float] = Field(
        ...,
        description='The view up (-vu) vector as an array of (x, y, z) values.',
        min_items=3,
        max_items=3
    )

    view_type: ViewType = ViewType.perspective

    h_size: float = Field(
        60,
        description='A number for the horizontal field of view in degrees (for '
        'all perspective projections including fisheye). For a parallel '
        'projection, this is the view width in world coordinates.'
    )

    v_size: float = Field(
        60,
        description='A number for the vertical field of view in degrees (for '
            'all perspective projections including fisheye). For a parallel '
            'projection, this is the view width in world coordinates.'
    )

    shift: float = Field(
        None,
        description='The view shift (-vs). This is the amount the actual '
        'image will be shifted to the right of the specified view. This '
        'option is useful for generating skewed perspectives or rendering '
        'an image a piece at a time. A value of 1 means that the rendered '
        'image starts just to the right of the normal view. A value of -1 '
        'would be to the left. Larger or fractional values are permitted '
        'as well.'
    )

    lift: float = Field(
        None,
        description='The view lift (-vl). This is the amount the actual '
        'image will be lifted up from the specified view. This '
        'option is useful for generating skewed perspectives or rendering '
        'an image a piece at a time. A value of 1 means that the rendered '
        'image starts just to the right of the normal view. A value of -1 '
        'would be to the left. Larger or fractional values are permitted '
        'as well.'
    )

    fore_clip: float = Field(
        None,
        description='View fore clip (-vo) at a distance from the view point.'
        'The plane will be perpendicular to the view direction for perspective '
        'and parallel view types. For fisheye view types, the clipping plane is '
        'actually a clipping sphere, centered on the view point with fore_clip radius. '
        'Objects in front of this imaginary surface will not be visible.'
    )

    aft_clip: float = Field(
        None,
        description='View aft clip (-va) at a distance from the view point.'
        'Like the view fore plane, it will be perpendicular to the view '
        'direction for perspective and parallel view types. For fisheye '
        'view types, the clipping plane is actually a clipping sphere, '
        'centered on the view point with radius val.'
    )



from honeybee_schema.radiance.asset import SensorGrid, View

import os
import json

# target folder where all of the samples live
root = os.path.dirname(os.path.dirname(__file__))
target_folder = os.path.join(root, 'samples', 'radiance_asset')


def test_sensor_grid_simple():
    file_path = os.path.join(target_folder, 'sensor_grid_simple.json')
    SensorGrid.parse_file(file_path)


def test_sensor_grid_detailed():
    file_path = os.path.join(target_folder, 'sensor_grid_detailed.json')
    SensorGrid.parse_file(file_path)


def test_view_perspective():
    file_path = os.path.join(target_folder, 'view_perspective.json')
    View.parse_file(file_path)


def test_view_parallel():
    file_path = os.path.join(target_folder, 'view_parallel.json')
    View.parse_file(file_path)



from honeybee_schema.radiance.properties import ApertureRadiancePropertiesAbridged, \
    DoorRadiancePropertiesAbridged, FaceRadiancePropertiesAbridged, \
    ShadeRadiancePropertiesAbridged, RoomRadiancePropertiesAbridged, \
    ModelRadianceProperties

import os
import json

# target folder where all of the samples live
root = os.path.dirname(os.path.dirname(__file__))
target_folder = os.path.join(root, 'samples', 'model')


file_path = os.path.join(target_folder, 'model_complete_office_floor.json')
with open(file_path) as json_file:
    office_model = json.load(json_file)


def test_model_radiance_properties():
    model_rad_props = office_model['properties']['radiance']
    ModelRadianceProperties.parse_obj(model_rad_props)


def test_room_radiance_properties():
    room_prop_abridged = office_model['rooms'][0]['properties']['radiance']
    RoomRadiancePropertiesAbridged.parse_obj(room_prop_abridged)


def test_face_radiance_properties():
    face_prop_abridged = office_model['rooms'][0]['faces'][0]['properties']['radiance']
    FaceRadiancePropertiesAbridged.parse_obj(face_prop_abridged)



"""State Schema"""
from pydantic import Field, constr
from typing import List
from .._base import NoExtraBaseModel
from ._base import IDdRadianceBaseModel

from ..geometry import Face3D


class StateGeometryAbridged(IDdRadianceBaseModel):
    """A single planar geometry that can be assigned to Radiance states."""

    type: constr(regex='^StateGeometryAbridged$') = 'StateGeometryAbridged'

    modifier: str = Field(
        default=None,
        description='A string for a Honeybee Radiance Modifier identifier '
                    '(default: None).'
        )

    modifier_direct: str = Field(
        default=None,
        description='A string for Honeybee Radiance Modifier identifiers to be used '
                    'in direct solar simulations and in isolation studies (assessing'
                    'the contribution of individual objects) (default: None).'
        )

    geometry: Face3D = Field(
        ...,
        description='A ladybug_geometry Face3D.'
    )


class RadianceShadeStateAbridged(NoExtraBaseModel):
    """RadianceShadeStateAbridged represents a single state for a dynamic Shade."""

    type: constr(regex='^RadianceShadeStateAbridged$') = 'RadianceShadeStateAbridged'

    modifier: str = Field(
        default=None,
        description='A Radiance Modifier identifier (default: None).'
    )

    modifier_direct: str = Field(
        default=None,
        description='A Radiance Modifier identifier (default: None).'
    )

    shades: List[StateGeometryAbridged] = Field(
        default=None,
        description='A list of StateGeometryAbridged objects (default: None).'
    )


class RadianceSubFaceStateAbridged(RadianceShadeStateAbridged):
    """RadianceSubFaceStateAbridged is an abridged state for a dynamic Aperture or Door.
    """

    type: constr(regex='^RadianceSubFaceStateAbridged$') = 'RadianceSubFaceStateAbridged'

    vmtx_geometry: Face3D = Field(
        default=None,
        description='A Face3D for the view matrix geometry (default: None).'
    )

    dmtx_geometry: Face3D = Field(
        default=None,
        description='A Face3D for the daylight matrix geometry (default: None).'
    )




# coding=utf-8
from __future__ import division

from honeybee_radiance.dynamic import RadianceShadeState, RadianceSubFaceState, \
    StateGeometry
from honeybee_radiance.modifier.material import Plastic, Glass, Trans, BSDF

from honeybee.shade import Shade
from honeybee.aperture import Aperture
from ladybug_geometry.geometry3d.pointvector import Point3D
from ladybug_geometry.geometry3d.face import Face3D

import os
import json


def shade_state_abridged_snow(directory):
    snow_mod = Plastic.from_single_reflectance('SnowMaterial', 0.7)
    snow = RadianceShadeState(snow_mod)

    dest_file = os.path.join(directory, 'shade_state_abridged_snow.json')
    with open(dest_file, 'w') as fp:
        json.dump(snow.to_dict(abridged=True), fp, indent=4)


def shade_state_abridged_tree_foliage(directory):
    shd1 = StateGeometry.from_vertices(
        'tree_foliage1', [[0, 0, 5], [2, 0, 5], [2, 2, 5], [0, 2, 5]])
    shd2 = StateGeometry.from_vertices(
        'tree_foliage2', [[0, 0, 5], [-2, 0, 5], [-2, 2, 5], [0, 2, 5]])
    trans1 = Glass.from_single_transmittance('TreeTrans1', 0.5)
    trans2 = Glass.from_single_transmittance('TreeTrans2', 0.27)
    shd1.modifier = trans1
    shd2.modifier = trans2
    tr4 = RadianceShadeState(shades=[shd1, shd2])

    dest_file = os.path.join(directory, 'shade_state_abridged_tree_foliage.json')
    with open(dest_file, 'w') as fp:
        json.dump(tr4.to_dict(abridged=True), fp, indent=4)


def aperture_state_abridged_electrochromic(directory):
    ecglass4 = Glass.from_single_transmittance('ElectrochromicGlass4', 0.01)
    tint4 = RadianceSubFaceState(ecglass4)

    dest_file = os.path.join(directory, 'aperture_state_abridged_electrochromic.json')
    with open(dest_file, 'w') as fp:
        json.dump(tint4.to_dict(abridged=True), fp, indent=4)


def aperture_state_abridged_shades(directory):
    mod = Trans('DiffusingShade', 0.7, 0.7, 0.7, 0.01, 0, 0.45, 0.01)
    mod_dir = Glass.from_single_transmittance('DiffusingShadeDirect', 0.03)
    mod_shd = Plastic('ShadeMat', 0.65, 0.65, 0.65)
    pts_1 = (Point3D(0, 0, 0), Point3D(2, 0, 0), Point3D(2, 2, 0), Point3D(0, 2, 0))
    pts_2 = (Point3D(0, 0, 2), Point3D(2, 0, 0), Point3D(2, 2, 2), Point3D(0, 2, 2))
    shade1 = StateGeometry('RectangleShade1', Face3D(pts_1))
    shade2 = StateGeometry('RectangleShade2', Face3D(pts_2))
    shade1.modifier = mod_shd
    shade2.modifier = mod_shd
    rad_state = RadianceSubFaceState(mod, [shade1, shade2])
    rad_state.modifier_direct = mod_dir

    dest_file = os.path.join(directory, 'aperture_state_abridged_shades.json')
    with open(dest_file, 'w') as fp:
        json.dump(rad_state.to_dict(abridged=True), fp, indent=4)


def aperture_state_abridged_bsdf(directory):
    relative_path = './scripts/bsdf/klemsfull.xml'
    mod = BSDF(relative_path)
    shd_state = RadianceSubFaceState(mod)

    pts = (Point3D(0, 0, 0), Point3D(0, 0, 3), Point3D(1, 0, 3), Point3D(1, 0, 0))
    ap = Aperture('TestWindow', Face3D(pts))
    ap.properties.radiance.dynamic_group_identifier = 'DynamicShdWindow'
    ap.properties.radiance.states = [shd_state]
    shd_state.gen_geos_from_tmtx_thickness(0.05)

    dest_file = os.path.join(directory, 'aperture_state_abridged_bsdf.json')
    with open(dest_file, 'w') as fp:
        json.dump(shd_state.to_dict(abridged=True), fp, indent=4)


# run all functions within the file
master_dir = os.path.split(os.path.dirname(__file__))[0]
sample_directory = os.path.join(master_dir, 'samples', 'state')

shade_state_abridged_snow(sample_directory)
shade_state_abridged_tree_foliage(sample_directory)
aperture_state_abridged_electrochromic(sample_directory)
aperture_state_abridged_shades(sample_directory)
aperture_state_abridged_bsdf(sample_directory)



import os
from honeybee_schema.radiance.state import RadianceSubFaceStateAbridged, \
    RadianceShadeStateAbridged


# target folder where all of the samples live
root = os.path.dirname(os.path.dirname(__file__))
target_folder = os.path.join(root, 'samples', 'state')


def test_aperture_state_abridged_bsdf():
    file_path = os.path.join(target_folder, 'aperture_state_abridged_bsdf.json')
    RadianceSubFaceStateAbridged.parse_file(file_path)


def test_aperture_state_abridged_electrochromic():
    file_path = os.path.join(target_folder, 'aperture_state_abridged_electrochromic.json')
    RadianceSubFaceStateAbridged.parse_file(file_path)


def test_aperture_state_abridged_shades():
    file_path = os.path.join(target_folder, 'aperture_state_abridged_shades.json')
    RadianceSubFaceStateAbridged.parse_file(file_path)


def test_shade_state_abridged_snow():
    file_path = os.path.join(target_folder, 'shade_state_abridged_snow.json')
    RadianceShadeStateAbridged.parse_file(file_path)


def test_shade_state_abridged_tree_foliage():
    file_path = os.path.join(target_folder, 'shade_state_abridged_tree_foliage.json')
    RadianceShadeStateAbridged.parse_file(file_path)




"""Properties Schema"""
from pydantic import Field, constr
from typing import List, Union

from .modifier import _REFERENCE_UNION_MODIFIERS
from .modifierset import ModifierSet, ModifierSetAbridged
from .._base import NoExtraBaseModel

from honeybee_schema.radiance.state import RadianceShadeStateAbridged, \
    RadianceSubFaceStateAbridged


class _PropertiesBaseAbridged(NoExtraBaseModel):
    """Base class of Abridged Radiance Properties."""

    modifier: str = Field(
        default=None,
        description='A string for a Honeybee Radiance Modifier (default: None).'
        )

    modifier_blk: str = Field(
        default=None,
        description='A string for a Honeybee Radiance Modifier to be used '
                    'in direct solar simulations and in isolation studies (assessing'
                    'the contribution of individual objects) (default: None).'
        )


class ApertureRadiancePropertiesAbridged(_PropertiesBaseAbridged):
    """Radiance Properties for Honeybee Aperture Abridged."""

    type: constr(regex='^ApertureRadiancePropertiesAbridged$') = \
        'ApertureRadiancePropertiesAbridged'

    dynamic_group_identifier: str = Field(
        default=None,
        description="An optional string to note the dynamic group ' \
            'to which the Aperture is a part of. Apertures sharing the same ' \
            'dynamic_group_identifier will have their states change in unison. ' \
            'If None, the Aperture is assumed to be static. (default: None)."
    )

    states: List[RadianceSubFaceStateAbridged] = Field(
        default=None,
        description="An optional list of abridged states (default: None)."
    )


class DoorRadiancePropertiesAbridged(_PropertiesBaseAbridged):
    """Radiance Properties for Honeybee Door Abridged."""

    type: constr(regex='^DoorRadiancePropertiesAbridged$') = \
        'DoorRadiancePropertiesAbridged'

    dynamic_group_identifier: str = Field(
        default=None,
        description="An optional string to note the dynamic group ' \
            'to which the Door is a part of. Doors sharing the same ' \
            'dynamic_group_identifier will have their states change in unison. ' \
            'If None, the Door is assumed to be static. (default: None)."
    )

    states: List[RadianceSubFaceStateAbridged] = Field(
        default=None,
        description="An optional list of abridged states (default: None)."
    )


class FaceRadiancePropertiesAbridged(_PropertiesBaseAbridged):
    """Radiance Properties for Honeybee Face Abridged."""

    type: constr(regex='^FaceRadiancePropertiesAbridged$') = \
        'FaceRadiancePropertiesAbridged'


class ShadeRadiancePropertiesAbridged(_PropertiesBaseAbridged):
    """Radiance Properties for Honeybee Shade Abridged."""

    type: constr(regex='^ShadeRadiancePropertiesAbridged$') = \
        'ShadeRadiancePropertiesAbridged'

    dynamic_group_identifier: str = Field(
        default=None,
        description="An optional string to note the dynamic group ' \
            'to which the Shade is a part of. Shades sharing the same ' \
            'dynamic_group_identifier will have their states change in unison. ' \
            'If None, the Shade is assumed to be static. (default: None)."
    )

    states: List[RadianceShadeStateAbridged] = Field(
        default=None,
        description="An optional list of abridged states (default: None)."
    )


class RoomRadiancePropertiesAbridged(NoExtraBaseModel):
    """Abridged Radiance Properties for Honeybee Room."""

    type: constr(regex='^RoomRadiancePropertiesAbridged$') = \
        'RoomRadiancePropertiesAbridged'

    modifier_set: str = Field(
        default=None,
        description='An identifier for a unique Room-Assigned ModifierSet '
                    '(default: None).'
        )


class ModelRadianceProperties(NoExtraBaseModel):
    """Radiance Properties for Honeybee Model."""

    type: constr(regex='^ModelRadianceProperties$') = 'ModelRadianceProperties'

    modifiers: List[_REFERENCE_UNION_MODIFIERS] = Field(
        default=None,
        description='A list of all unique modifiers in the model. '
                    'This includes modifiers across all Faces, Apertures, Doors, '
                    'Shades, Room ModifierSets, and the global_modifier_set '
                    '(default: None).'
        )

    modifier_sets: List[Union[ModifierSet, ModifierSetAbridged]] = Field(
        default=None,
        description='A list of all unique Room-Assigned ModifierSets in the Model '
                    '(default: None).'
        )

    global_modifier_set: str = Field(
        default=None,
        description='Identifier of a ModifierSet or ModifierSetAbridged object to be '
                    'used as as a default object for all unassigned objects in the '
                    'Model (default: None).'
        )




# coding=utf-8
from __future__ import division

from honeybee_radiance.sensor import Sensor
from honeybee_radiance.sensorgrid import SensorGrid
from honeybee_radiance.view import View

from honeybee.room import Room

import os
import json


def sensor_grid_simple(directory):
    sensors = [Sensor((0, 0, 0), (0, 0, 1)), Sensor((0, 0, 10), (0, 0, 1))]
    sg = SensorGrid('sg_1', sensors)
    dest_file = os.path.join(directory, 'sensor_grid_simple.json')
    with open(dest_file, 'w') as fp:
        json.dump(sg.to_dict(), fp, indent=4)


def sensor_grid_detailed(directory):
    room1 = Room.from_box('Tiny_House_Room', 5, 10, 3)
    sensor_grid, mesh = room1.properties.radiance.generate_sensor_grid(1, 1, 1)
    sensor_grid.display_name = 'Tiny House Sensor Grid'
    dest_file = os.path.join(directory, 'sensor_grid_detailed.json')
    with open(dest_file, 'w') as fp:
        json.dump(sensor_grid.to_dict(), fp, indent=4)


def view_perspective(directory):
    vw = View('test_view_perspective', (0, 0, 10), (0, 1, 0), (0, 0, 1))
    dest_file = os.path.join(directory, 'view_perspective.json')
    with open(dest_file, 'w') as fp:
        json.dump(vw.to_dict(), fp, indent=4)


def view_parallel(directory):
    vw = View('test_view', (0, 0, 10), (0, 1, 0), (0, 0, 1), 'l', 240, 300, -10, -25)
    dest_file = os.path.join(directory, 'view_parallel.json')
    with open(dest_file, 'w') as fp:
        json.dump(vw.to_dict(), fp, indent=4)


# run all functions within the file
master_dir = os.path.split(os.path.dirname(__file__))[0]
sample_directory = os.path.join(master_dir, 'samples', 'radiance_assets')

sensor_grid_simple(sample_directory)
sensor_grid_detailed(sample_directory)
view_perspective(sample_directory)
view_parallel(sample_directory)



from honeybee_schema.radiance.modifierset import ModifierSetAbridged, ModifierSet

import os

# target folder where all of the samples live
root = os.path.dirname(os.path.dirname(__file__))
target_folder = os.path.join(root, 'samples', 'modifier_set')


def test_modifierset_abridged_complete():
    file_path = os.path.join(target_folder, 'modifierset_abridged_complete.json')
    ModifierSetAbridged.parse_file(file_path)


def test_modifierset_abridged_partial_exterior():
    file_path = os.path.join(target_folder, 'modifierset_abridged_partial_exterior.json')
    ModifierSetAbridged.parse_file(file_path)


def test_modifierset_complete():
    file_path = os.path.join(target_folder, 'modifierset_complete.json')
    ModifierSet.parse_file(file_path)


def test_modifierset_partial_exterior():
    file_path = os.path.join(target_folder, 'modifierset_partial_exterior.json')
    ModifierSet.parse_file(file_path)



"""Geometry objects for model."""
from pydantic import Field, constr, conlist
from typing import List
from ._base import NoExtraBaseModel


class Plane(NoExtraBaseModel):

    type: constr(regex='^Plane$') = 'Plane'

    n: List[float] = Field(
        ...,
        description="Plane normal as 3 (x, y, z) values.",
        min_items=3,
        max_items=3
    )

    o: List[float] = Field(
        ...,
        description="Plane origin as 3 (x, y, z) values",
        min_items=3,
        max_items=3
    )

    x: List[float] = Field(
        default=None,
        description="Plane x-axis as 3 (x, y, z) values. If None, it is autocalculated.",
        min_items=3,
        max_items=3
    )


class Face3D(NoExtraBaseModel):
    """A single planar face in 3D space."""

    type: constr(regex='^Face3D$') = 'Face3D'

    boundary: List[conlist(float, min_items=3, max_items=3)] = Field(
        ...,
        min_items=3,
        description='A list of points representing the outer boundary vertices of '
            'the face. The list should include at least 3 points and each point '
            'should be a list of 3 (x, y, z) values.'
    )

    holes: List[conlist(conlist(float, min_items=3, max_items=3), min_items=3)] = Field(
        default=None,
        description='Optional list of lists with one list for each hole in the face.'
            'Each hole should be a list of at least 3 points and each point a list '
            'of 3 (x, y, z) values. If None, it will be assumed that there are no '
            'holes in the face.'
    )

    plane: Plane = Field(
        default=None,
        description='Optional Plane indicating the plane in which the face exists.'
            'If None, the plane will usually be derived from the boundary points.'
    )


"""Base class used by radiance schema objects."""
from pydantic import Field, validator
import re
from .._base import NoExtraBaseModel


class IDdRadianceBaseModel(NoExtraBaseModel):
    """Base class for all objects requiring a valid Radiance identifier."""

    identifier: str = Field(
        ...,
        description='Text string for a unique Radiance object. Must not contain spaces '
                    'or special characters. This will be used to identify the object '
                    'across a model and in the exported Radiance files.'
    )

    @validator('identifier')
    def valid_rad_string(cls, value):
        """Check that a string is valid for Radiance.

        This method is modified from the honeybee-core.typing.valid_rad_string method.
        """
        try:
            illegal_match = re.search(r'[^.A-Za-z0-9_-]', value)
        except TypeError:
            raise TypeError('Identifier must be a text string. Got {}: {}.'.format(
                type(value), value))
        assert illegal_match is None, \
            'Illegal character "{}" found in identifier'.format(illegal_match.group(0))
        assert len(value) > 0, \
            'Input identifier "{}" contains no characters.'.format(value)
        return value

    display_name: str = Field(
        default=None,
        description='Display name of the object with no character restrictions.'
    )


"""ModifierSet Schema"""
from pydantic import Field, constr
from .._base import NoExtraBaseModel
from ._base import IDdRadianceBaseModel
from .modifier import _REFERENCE_UNION_MODIFIERS


class BaseModifierSetAbridged(NoExtraBaseModel):
    """Base class for the abridged modifier sets assigned to Faces."""

    exterior_modifier: str = Field(
            default=None,
            description='Identifier for a radiance modifier object for faces with an '
                        ' Outdoors boundary condition.'
        )

    interior_modifier: str = Field(
            default=None,
            description='Identifier for a radiance modifier object for faces with a '
                        'boundary condition other than Outdoors.'
        )


class WallModifierSetAbridged(BaseModifierSetAbridged):
    """Abridged set containing radiance modifiers needed for a model's Walls."""

    type: constr(regex='^WallModifierSetAbridged$') = 'WallModifierSetAbridged'


class FloorModifierSetAbridged(BaseModifierSetAbridged):
    """Abridged set containing radiance modifiers needed for a model's Floors."""

    type: constr(regex='^FloorModifierSetAbridged$') = 'FloorModifierSetAbridged'


class RoofCeilingModifierSetAbridged(BaseModifierSetAbridged):
    """Abridged set containing radiance modifiers needed for a model's Roofs."""

    type: constr(regex='^RoofCeilingModifierSetAbridged$') = \
        'RoofCeilingModifierSetAbridged'


class ShadeModifierSetAbridged(BaseModifierSetAbridged):
    """Abridged set containing radiance modifiers needed for a model's Shade."""

    type: constr(regex='^ShadeModifierSetAbridged$') = 'ShadeModifierSetAbridged'


class ApertureModifierSetAbridged(NoExtraBaseModel):
    """Abridged set containing radiance modifiers needed for a model's Apertures."""

    type: constr(regex='^ApertureModifierSetAbridged$') = 'ApertureModifierSetAbridged'

    window_modifier: str = Field(
            default=None,
            description='Identifier of modifier object for apertures with an Outdoors '
                        'boundary condition, False is_operable property, '
                        'and Wall parent Face.'
        )

    interior_modifier: str = Field(
            default=None,
            description='Identifier of modifier object for apertures with a Surface '
                        'boundary condition.'
        )

    skylight_modifier: str = Field(
            default=None,
            description='Identifier of modifier object for apertures with an Outdoors '
                        'boundary condition, False is_operable property, and a '
                        'RoofCeiling or Floor face type for their parent face.'
        )

    operable_modifier: str = Field(
            default=None,
            description='Identifier of modifier object for apertures with an Outdoors '
                        'boundary condition and a True is_operable property.'
        )


class DoorModifierSetAbridged(BaseModifierSetAbridged):
    """Abridged set containing radiance modifiers needed for a model's Doors."""

    type: constr(regex='^DoorModifierSetAbridged$') = 'DoorModifierSetAbridged'

    interior_glass_modifier: str = Field(
            default=None,
            description='Identifier of modifier object for glass with a Surface '
                        'boundary condition.'
        )

    exterior_glass_modifier: str = Field(
            default=None,
            description='Identifier of modifier object for glass with an Outdoors '
                        'boundary condition.'
        )

    overhead_modifier: str = Field(
            default=None,
            description='Identifier of a modifier object for doors with an '
                        'Outdoors boundary condition and a RoofCeiling or Floor '
                        'face type for their parent face.'
        )


class ModifierSetAbridged(IDdRadianceBaseModel):
    """Abridged set containing all modifiers needed to create a radiance model."""

    type: constr(regex='^ModifierSetAbridged$') = 'ModifierSetAbridged'

    wall_set: WallModifierSetAbridged = Field(
        default=None,
        description='Optional WallModifierSet object for this '
                    'ModifierSet (default: None).'
    )

    floor_set: FloorModifierSetAbridged = Field(
        default=None,
        description='Optional FloorModifierSet object for '
                    'this ModifierSet (default: None).'
    )

    roof_ceiling_set: RoofCeilingModifierSetAbridged = Field(
        default=None,
        description='Optional RoofCeilingModifierSet object for this '
                    'ModifierSet (default: None).'
    )

    aperture_set: ApertureModifierSetAbridged = Field(
        default=None,
        description='Optional ApertureModifierSet object for this '
                    'ModifierSet (default: None).'
    )

    door_set: DoorModifierSetAbridged = Field(
        default=None,
        description='Optional DoorModifierSet object for this '
                    'ModifierSet (default: None).'
    )

    shade_set: ShadeModifierSetAbridged = Field(
        default=None,
        description='Optional ShadeModifierSet object for this '
                    'ModifierSet (default: None).'
    )

    air_boundary_modifier: str = Field(
        default=None,
        description='Optional Modifier to be used for all Faces '
                    'with an AirBoundary face type. If None, it will be the '
                    'honyebee generic air wall modifier.'
    )


class BaseModifierSet(NoExtraBaseModel):
    """Base class for the modifier sets assigned to Faces."""

    exterior_modifier: _REFERENCE_UNION_MODIFIERS = Field(
            default=None,
            description='A radiance modifier object for faces with an Outdoors boundary '
                        'condition.'
        )

    interior_modifier: _REFERENCE_UNION_MODIFIERS = Field(
            default=None,
            description='A radiance modifier object for faces with a boundary condition '
                        'other than Outdoors.'
        )


class WallModifierSet(BaseModifierSet):
    """Set containing radiance modifiers needed for a model's Walls."""

    type: constr(regex='^WallModifierSet$') = 'WallModifierSet'


class FloorModifierSet(BaseModifierSet):
    """Set containing radiance modifiers needed for a model's Floors."""

    type: constr(regex='^FloorModifierSet$') = 'FloorModifierSet'


class RoofCeilingModifierSet(BaseModifierSet):
    """Set containing radiance modifiers needed for a model's roofs."""

    type: constr(regex='^RoofCeilingModifierSet$') = 'RoofCeilingModifierSet'


class ShadeModifierSet(BaseModifierSet):
    """Set containing radiance modifiers needed for a model's Shade."""

    type: constr(regex='^ShadeModifierSet$') = 'ShadeModifierSet'


class ApertureModifierSet(NoExtraBaseModel):
    """Set containing radiance modifiers needed for a model's Apertures."""

    type: constr(regex='^ApertureModifierSet$') = 'ApertureModifierSet'

    window_modifier: _REFERENCE_UNION_MODIFIERS = Field(
            default=None,
            description='A modifier object for apertures with an Outdoors '
                        'boundary condition, False is_operable property, '
                        'and Wall parent Face.'
        )

    interior_modifier: _REFERENCE_UNION_MODIFIERS = Field(
            default=None,
            description='A modifier object for apertures with a Surface '
                        'boundary condition.'
        )

    skylight_modifier: _REFERENCE_UNION_MODIFIERS = Field(
            default=None,
            description='A modifier object for apertures with an Outdoors '
                        'boundary condition, False is_operable property, and a '
                        'RoofCeiling or Floor face type for their parent face.'
        )

    operable_modifier: _REFERENCE_UNION_MODIFIERS = Field(
            default=None,
            description='A modifier object for apertures with an Outdoors boundary '
                        'condition and a True is_operable property.'
        )


class DoorModifierSet(BaseModifierSet):
    """Set containing radiance modifiers needed for a model's Doors."""

    type: constr(regex='^DoorModifierSet$') = 'DoorModifierSet'

    interior_glass_modifier: _REFERENCE_UNION_MODIFIERS = Field(
            default=None,
            description='A modifier object for glass with a Surface '
                        'boundary condition.'
        )

    exterior_glass_modifier: _REFERENCE_UNION_MODIFIERS = Field(
            default=None,
            description='A modifier object for glass with an Outdoors '
                        'boundary condition.'
        )

    overhead_modifier: _REFERENCE_UNION_MODIFIERS = Field(
            default=None,
            description='A window modifier object for doors with an Outdoors boundary '
                        'condition and a RoofCeiling or Floor face type for their '
                        'parent face.'
        )


class ModifierSet(IDdRadianceBaseModel):
    """Set containing all radiance modifiers needed to create a radiance model."""

    type: constr(regex='^ModifierSet$') = 'ModifierSet'

    wall_set: WallModifierSet = Field(
        default=None,
        description='An optional WallModifierSet object for this ModifierSet. '
                    '(default: None).'
    )

    floor_set: FloorModifierSet = Field(
        default=None,
        description='An optional FloorModifierSet object for this ModifierSet. '
                    '(default: None).'
    )

    roof_ceiling_set: RoofCeilingModifierSet = Field(
        default=None,
        description='An optional RoofCeilingModifierSet object for this ModifierSet. '
                    '(default: None).'
    )

    aperture_set: ApertureModifierSet = Field(
        default=None,
        description='An optional ApertureModifierSet object for this ModifierSet. '
                    '(default: None).'
    )

    door_set: DoorModifierSet = Field(
        default=None,
        description='An optional DoorModifierSet object for this ModifierSet. '
                    '(default: None).'
    )

    shade_set: ShadeModifierSet = Field(
        default=None,
        description='An optional ShadeModifierSet object for this ModifierSet. '
                    '(default: None).'
    )

    air_boundary_modifier: _REFERENCE_UNION_MODIFIERS = Field(
        default=None,
        description='An optional Modifier to be used for all Faces with an AirBoundary '
                    'face type. If None, it will be the honyebee generic air wall '
                    'modifier.'
    )